# graphql-api-koa

[![Build status](https://travis-ci.org/jaydenseric/graphql-api-koa.svg)](https://travis-ci.org/jaydenseric/graphql-api-koa) [![npm version](https://img.shields.io/npm/v/graphql-api-koa.svg)](https://npm.im/graphql-api-koa)

GraphQL API Koa middleware.

## Setup

To install [`graphql-api-koa`](https://npm.im/graphql-api-koa) and [`graphql`](https://npm.im/graphql) from [npm](https://npmjs.com) run:

```sh
npm install graphql-api-koa graphql
```

See the [execute middleware](#execute) examples to get started.

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

- [errorHandler](#errorhandler)
  - [Examples](#examples)
- [execute](#execute)
  - [Parameters](#parameters)
  - [Examples](#examples-1)
- [Types](#types)
  - [ExecuteOptions](#executeoptions)
    - [Properties](#properties)
    - [Examples](#examples-2)
  - [MiddlewareOptionsOverride](#middlewareoptionsoverride)
    - [Parameters](#parameters-1)
    - [Examples](#examples-3)

### errorHandler

Creates Koa middleware to handle errors. Use this as the first to catch all errors for [a correctly formated GraphQL response](http://facebook.github.io/graphql/October2016/#sec-Errors). When intentionally throwing an error, create it with `status` and `expose` properties using [http-errors](https://npm.im/http-errors) or the response will be a generic 500 error for security.

#### Examples

_How to throw an error determining the response._

```javascript
import Koa from 'koa'
import bodyParser from 'koa-bodyparser'
import { errorHandler, execute } from 'graphql-api-koa'
import createError from 'http-errors'
import schema from './schema'

const app = new Koa()
  .use(errorHandler())
  .use(async (ctx, next) => {
    if (
      // It’s Saturday.
      new Date().getDay() === 6
    )
      throw createError(503, 'No work on the sabbath.', { expose: true })

    await next()
  })
  .use(bodyParser())
  .use(execute({ schema }))
```

Returns **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** Koa middleware.

### execute

Creates Koa middleware to execute GraphQL. Use after the [errorHandler](#errorhandler) and [body parser](https://npm.im/koa-bodyparser) middleware.

#### Parameters

- `options` **[ExecuteOptions](#executeoptions)** Options.

#### Examples

_A basic GraphQL API._

```javascript
import Koa from 'koa'
import bodyParser from 'koa-bodyparser'
import { errorHandler, execute } from 'graphql-api-koa'
import schema from './schema'

const app = new Koa()
  .use(errorHandler())
  .use(bodyParser())
  .use(execute({ schema }))
```

Returns **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** Koa middleware.

### Types

The following types are for documentation only and are not exported.

#### ExecuteOptions

GraphQL [execute](#execute) Koa middleware options.

Type: [Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)

##### Properties

- `schema` **module:graphql.GraphQLSchema** GraphQL schema.
- `rootValue` **any?** Value passed to the first resolver.
- `contextValue` **any?** Execution context (usually an object) passed to resolvers.
- `fieldResolver` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)?** Custom default field resolver.
- `override` **[MiddlewareOptionsOverride](#middlewareoptionsoverride)?** Override any [ExecuteOptions](#executeoptions) (except `override`) per request.

##### Examples

_[execute](#execute) middleware options that sets the schema once but populates the user in the GraphQL context from the Koa context each request._

```javascript
import schema from './schema'

const executeOptions = {
  schema,
  override: ctx => ({
    contextValue: {
      user: ctx.state.user
    }
  })
}
```

#### MiddlewareOptionsOverride

Per-request Koa middleware options override.

Type: [Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)

##### Parameters

- `context` **module:koa.Context** Koa context.

##### Examples

_An [execute](#execute) middleware options override that populates the user in the GraphQL context from the Koa request context._

```javascript
const executeOptionsOverride = ctx => ({
  contextValue: {
    user: ctx.state.user
  }
})
```

Returns **[Object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** Options.
